@@ -1,4 +1,4 @@
-# Copyright (c) 2015-2016 Matthias Geier
+# Copyright (c) 2015-2017 Matthias Geier
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,7 @@
 http://nbsphinx.rtfd.org/
 
 """
-__version__ = '0.2.13'
+__version__ = '0.2.14'
 
 import copy
 import json
@@ -141,8 +141,9 @@ RST_TEMPLATE = """
 {%- elif datatype in ['text/latex'] %}
 
     .. math::
+        :nowrap:
 
-{{ output.data['text/latex'] | strip_dollars | indent | indent }}
+{{ output.data['text/latex'] | indent | indent }}
 {%- elif datatype == 'text/html' %}
 
     .. raw:: html
@@ -175,10 +176,14 @@ RST_TEMPLATE = """
 
     .. raw:: latex
 
-        % This comment is needed to force a line break for adjacent ANSI cells
+        %
         \\begin{OriginalVerbatim}[commandchars=\\\\\\{\\}]
 {{ output.data[datatype] | ansi2latex | indent | indent }}
         \\end{OriginalVerbatim}
+        % The following \\relax is needed to avoid problems with adjacent ANSI
+        % cells and some other stuff (e.g. bullet lists) following ANSI cells.
+        % See https://github.com/sphinx-doc/sphinx/issues/3594
+        \\relax
 {% else %}
 
     .. nbwarning:: Data type cannot be displayed: {{ datatype }}
@@ -225,6 +230,7 @@ RST_TEMPLATE = """
 
 .. raw:: latex
 
+    %
 {{ cell.source | indent }}
 {%- elif raw_mimetype == 'text/html' %}
 .. raw:: html
@@ -233,6 +239,7 @@ RST_TEMPLATE = """
 {%- elif raw_mimetype == 'text/latex' %}
 .. raw:: latex
 
+    %
 {{ cell.source | indent }}
 {%- elif raw_mimetype == 'text/markdown' %}
 {{ cell.source | markdown2rst }}
@@ -477,6 +484,32 @@ CSS_STRING_READTHEDOCS = """
 }
 """
 
+CSS_STRING_CLOUD = """
+/* CSS overrides for cloud theme */
+
+/* nicer titles and more space for info and warning logos */
+
+div.admonition > .first {
+    background: rgba(0, 0, 0, .05);
+    margin: .5em -1em;
+    margin-top: -.5em !important;
+    padding: .5em .5em .5em 2.65em;
+}
+
+/* indent single paragraph */
+div.admonition {
+    text-indent: 20px;
+}
+/* don't indent multiple paragraphs */
+div.admonition > p {
+    text-indent: 0;
+}
+/* remove excessive padding */
+div.admonition.inline-title p.admonition-title {
+    padding-left: .2em;
+}
+"""
+
 
 class Exporter(nbconvert.RSTExporter):
     """Convert Jupyter notebooks to reStructuredText.
@@ -489,10 +522,11 @@ class Exporter(nbconvert.RSTExporter):
 
     """
 
-    def __init__(self, execute='auto', execute_arguments=[],
+    def __init__(self, execute='auto', kernel_name='', execute_arguments=[],
                  allow_errors=False, timeout=30, codecell_lexer='none'):
         """Initialize the Exporter."""
         self._execute = execute
+        self._kernel_name = kernel_name
         self._execute_arguments = execute_arguments
         self._allow_errors = allow_errors
         self._timeout = timeout
@@ -500,9 +534,10 @@ class Exporter(nbconvert.RSTExporter):
         loader = jinja2.DictLoader({'nbsphinx-rst.tpl': RST_TEMPLATE})
         super(Exporter, self).__init__(
             template_file='nbsphinx-rst.tpl', extra_loaders=[loader],
-            config= traitlets.config.Config(
+            config=traitlets.config.Config(
                 {'HighlightMagicsPreprocessor': {'enabled': True}}),
             filters={
+                'convert_pandoc': convert_pandoc,
                 'markdown2rst': markdown2rst,
                 'get_empty_lines': _get_empty_lines,
                 'extract_toctree': _extract_toctree,
@@ -537,6 +572,7 @@ class Exporter(nbconvert.RSTExporter):
                 'allow_errors', self._allow_errors)
             timeout = nbsphinx_metadata.get('timeout', self._timeout)
             pp = nbconvert.preprocessors.ExecutePreprocessor(
+                kernel_name=self._kernel_name,
                 extra_arguments=self._execute_arguments,
                 allow_errors=allow_errors, timeout=timeout)
             nb, resources = pp.preprocess(nb, resources)
@@ -587,6 +623,7 @@ class NotebookParser(rst.Parser):
 
         exporter = Exporter(
             execute=env.config.nbsphinx_execute,
+            kernel_name=env.config.nbsphinx_kernel_name,
             execute_arguments=env.config.nbsphinx_execute_arguments,
             allow_errors=env.config.nbsphinx_allow_errors,
             timeout=env.config.nbsphinx_timeout,
@@ -762,6 +799,18 @@ class NbWarning(_NbAdmonition):
     _class = 'warning'
 
 
+def convert_pandoc(text, from_format, to_format):
+    """Simple wrapper for markdown2rst.
+
+    In nbconvert version 5.0, the use of markdown2rst in the RST
+    template was replaced by the new filter function convert_pandoc.
+
+    """
+    if from_format != 'markdown' and to_format != 'rst':
+        raise ValueError('Unsupported conversion')
+    return markdown2rst(text)
+
+
 def markdown2rst(text):
     """Convert a Markdown string to reST via pandoc.
 
@@ -778,7 +827,8 @@ def markdown2rst(text):
                 't': 'Math',
                 'c': [
                     {'t': 'DisplayMath', 'c': []},
-                    obj['c'][1],
+                    # Special marker characters are removed below:
+                    '\x0e:nowrap:\x0f\n\n' + obj['c'][1],
                 ]
             }]
         return obj
@@ -788,8 +838,8 @@ def markdown2rst(text):
         return json.dumps(json_data)
 
     rststring = pandoc(text, 'markdown', 'rst', filter_func=rawlatex2math)
-    return re.sub(r'^(\s*)\.\. math::$',
-                  r'\1.. math::\1   :nowrap:',
+    return re.sub(r'^\n( *)\x0e:nowrap:\x0f$',
+                  r'\1:nowrap:',
                   rststring,
                   flags=re.MULTILINE)
 
@@ -1113,6 +1163,8 @@ def html_page_context(app, pagename, templatename, context, doctree):
         style += CSS_STRING % app.config
     if doctree and app.config.html_theme in ('sphinx_rtd_theme', 'julia'):
         style += CSS_STRING_READTHEDOCS
+    if doctree and app.config.html_theme in ('cloud', 'redcloud'):
+        style += CSS_STRING_CLOUD
     if style:
         context['body'] = '\n<style>' + style + '</style>\n' + context['body']
 
@@ -1260,6 +1312,7 @@ def setup(app):
     _add_notebook_parser(app)
 
     app.add_config_value('nbsphinx_execute', 'auto', rebuild='env')
+    app.add_config_value('nbsphinx_kernel_name', '', rebuild='env')
     app.add_config_value('nbsphinx_execute_arguments', [], rebuild='env')
     app.add_config_value('nbsphinx_allow_errors', False, rebuild='')
     app.add_config_value('nbsphinx_timeout', 30, rebuild='')
